@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_MULTI_FILE_ALLOCATION_ITERATOR_H_
-#define _D_MULTI_FILE_ALLOCATION_ITERATOR_H_
+#ifndef D_MULTI_FILE_ALLOCATION_ITERATOR_H
+#define D_MULTI_FILE_ALLOCATION_ITERATOR_H
 
 #include "FileAllocationIterator.h"
 #include <deque>
@@ -46,11 +46,10 @@ class DiskWriterEntry;
 class MultiFileAllocationIterator:public FileAllocationIterator
 {
 private:
-  MultiDiskAdaptor* _diskAdaptor;
-  std::deque<SharedHandle<DiskWriterEntry> > _entries;
-  SharedHandle<FileAllocationIterator> _fileAllocationIterator;
-  off_t _offset;
-
+  MultiDiskAdaptor* diskAdaptor_;
+  std::deque<SharedHandle<DiskWriterEntry> > entries_;
+  SharedHandle<FileAllocationIterator> fileAllocationIterator_;
+  off_t offset_;
 public:
   MultiFileAllocationIterator(MultiDiskAdaptor* diskAdaptor);
 
@@ -62,13 +61,12 @@ public:
 
   virtual off_t getCurrentLength();
 
-  virtual uint64_t getTotalLength();
+  virtual off_t getTotalLength();
 
-  const std::deque<SharedHandle<DiskWriterEntry> >& getDiskWriterEntries() const;
+  const std::deque<SharedHandle<DiskWriterEntry> >&
+  getDiskWriterEntries() const;
 };
 
-typedef SharedHandle<MultiFileAllocationIterator> MultiFileAllocationIteratorHandle;
-
 } // namespace aria2
 
-#endif // _D_MULTI_FILE_ALLOCATION_ITERATOR_H_
+#endif // D_MULTI_FILE_ALLOCATION_ITERATOR_H
