@@ -60,6 +60,7 @@
 #include "fmt.h"
 #include "SocketCore.h"
 #include "OutputFile.h"
+#include "UriListParser.h"
 #ifdef ENABLE_SSL
 # include "TLSContext.h"
 #endif // ENABLE_SSL
@@ -99,9 +100,9 @@ namespace {
 ares_addr_node* parseAsyncDNSServers(const std::string& serversOpt)
 {
   std::vector<std::string> servers;
-  util::split(serversOpt,
+  util::split(serversOpt.begin(), serversOpt.end(),
               std::back_inserter(servers),
-              A2STR::COMMA_C,
+              ',',
               true /* doStrip */);
   ares_addr_node root;
   root.next = 0;
@@ -120,12 +121,12 @@ ares_addr_node* parseAsyncDNSServers(const std::string& serversOpt)
       node->next = 0;
       node->family = res->ai_family;
       if(node->family == AF_INET) {
-        struct sockaddr_in* in =
-          reinterpret_cast<struct sockaddr_in*>(res->ai_addr);
+        sockaddr_in* in =
+          &reinterpret_cast<sockaddr_union*>(res->ai_addr)->in;
         memcpy(&node->addr.addr4, &(in->sin_addr), 4);
       } else {
-        struct sockaddr_in6* in =
-          reinterpret_cast<struct sockaddr_in6*>(res->ai_addr);
+        sockaddr_in6* in =
+          &reinterpret_cast<sockaddr_union*>(res->ai_addr)->in6;
         memcpy(&node->addr.addr6, &(in->sin6_addr), 16);
       }
       tail->next = node;
@@ -143,11 +144,13 @@ MultiUrlRequestInfo::MultiUrlRequestInfo
 (const std::vector<SharedHandle<RequestGroup> >& requestGroups,
  const SharedHandle<Option>& op,
  const SharedHandle<StatCalc>& statCalc,
- const SharedHandle<OutputFile>& summaryOut)
+ const SharedHandle<OutputFile>& summaryOut,
+ const SharedHandle<UriListParser>& uriListParser)
   : requestGroups_(requestGroups),
     option_(op),
     statCalc_(statCalc),
-    summaryOut_(summaryOut)
+    summaryOut_(summaryOut),
+    uriListParser_(uriListParser)
 {}
 
 MultiUrlRequestInfo::~MultiUrlRequestInfo() {}
@@ -235,6 +238,9 @@ error_code::Value MultiUrlRequestInfo::execute()
         (option_->getAsInt(PREF_SERVER_STAT_TIMEOUT));
     }
     e->setStatCalc(statCalc_);
+    if(uriListParser_) {
+      e->getRequestGroupMan()->setUriListParser(uriListParser_);
+    }
 #ifdef SIGHUP
     util::setGlobalSignalHandler(SIGHUP, handler, 0);
 #endif // SIGHUP
@@ -271,10 +277,10 @@ error_code::Value MultiUrlRequestInfo::execute()
     if(!option_->blank(PREF_SAVE_SESSION)) {
       const std::string& filename = option_->get(PREF_SAVE_SESSION);
       if(sessionSerializer.save(filename)) {
-        A2_LOG_NOTICE(fmt("Serialized session to '%s' successfully.",
+        A2_LOG_NOTICE(fmt(_("Serialized session to '%s' successfully."),
                           filename.c_str()));
       } else {
-        A2_LOG_NOTICE(fmt("Failed to serialize session to '%s'.",
+        A2_LOG_NOTICE(fmt(_("Failed to serialize session to '%s'."),
                           filename.c_str()));
       }
     }
