@@ -39,40 +39,17 @@
 
 #include <vector>
 
-#include "NameMatchOptionHandler.h"
+#include "AbstractOptionHandler.h"
 
 namespace aria2 {
 
 class Option;
+class Pref;
 
-class NullOptionHandler : public OptionHandler {
-private:
-  int id_;
+class BooleanOptionHandler : public AbstractOptionHandler {
 public:
-  virtual ~NullOptionHandler();
-  virtual bool canHandle(const std::string& optName);
-  virtual void parse(Option& option, const std::string& arg);
-  virtual bool hasTag(const std::string& tag) const;
-  virtual void addTag(const std::string& tag);
-  virtual std::string toTagString() const;
-  virtual const std::string& getName() const;
-  virtual const std::string& getDescription() const;
-  virtual const std::string& getDefaultValue() const;
-  virtual std::string createPossibleValuesString() const;
-  virtual bool isHidden() const;
-  virtual void hide();
-  virtual OptionHandler::ARG_TYPE getArgType() const;
-  virtual int getOptionID() const;
-  virtual void setOptionID(int id);
-  virtual char getShortName() const;
-  virtual bool getEraseAfterParse() const;
-  virtual void setEraseAfterParse(bool eraseAfterParse);
-};
-
-class BooleanOptionHandler : public NameMatchOptionHandler {
-public:
-  BooleanOptionHandler(const std::string& optName,
-                       const std::string& description = NO_DESCRIPTION,
+  BooleanOptionHandler(const Pref* pref,
+                       const char* description = NO_DESCRIPTION,
                        const std::string& defaultValue = NO_DEFAULT_VALUE,
                        OptionHandler::ARG_TYPE argType = OptionHandler::REQ_ARG,
                        char shortName = 0);
@@ -81,13 +58,13 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class IntegerRangeOptionHandler : public NameMatchOptionHandler {
+class IntegerRangeOptionHandler : public AbstractOptionHandler {
 private:
   int32_t min_;
   int32_t max_;
 public:
-  IntegerRangeOptionHandler(const std::string& optName,
-                            const std::string& description,
+  IntegerRangeOptionHandler(const Pref* pref,
+                            const char* description,
                             const std::string& defaultValue,
                             int32_t min, int32_t max,
                             char shortName = 0);
@@ -96,13 +73,13 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class NumberOptionHandler : public NameMatchOptionHandler {
+class NumberOptionHandler : public AbstractOptionHandler {
 private:
   int64_t min_;
   int64_t max_;
 public:
-  NumberOptionHandler(const std::string& optName,
-                      const std::string& description = NO_DESCRIPTION,
+  NumberOptionHandler(const Pref* pref,
+                      const char* description = NO_DESCRIPTION,
                       const std::string& defaultValue = NO_DEFAULT_VALUE,
                       int64_t min = -1,
                       int64_t max = -1,
@@ -116,8 +93,8 @@ public:
 
 class UnitNumberOptionHandler : public NumberOptionHandler {
 public:
-  UnitNumberOptionHandler(const std::string& optName,
-                          const std::string& description = NO_DESCRIPTION,
+  UnitNumberOptionHandler(const Pref* pref,
+                          const char* description = NO_DESCRIPTION,
                           const std::string& defaultValue = NO_DEFAULT_VALUE,
                           int64_t min = -1,
                           int64_t max = -1,
@@ -126,13 +103,13 @@ public:
   virtual void parseArg(Option& option, const std::string& optarg);
 };
 
-class FloatNumberOptionHandler : public NameMatchOptionHandler {
+class FloatNumberOptionHandler : public AbstractOptionHandler {
 private:
   double min_;
   double max_;
 public:
-  FloatNumberOptionHandler(const std::string& optName,
-                           const std::string& description = NO_DESCRIPTION,
+  FloatNumberOptionHandler(const Pref* pref,
+                           const char* description = NO_DESCRIPTION,
                            const std::string& defaultValue = NO_DEFAULT_VALUE,
                            double min = -1, double max = -1,
                            char shortName = 0);
@@ -141,12 +118,12 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class DefaultOptionHandler : public NameMatchOptionHandler {
+class DefaultOptionHandler : public AbstractOptionHandler {
 private:
   std::string possibleValuesString_;
 public:
-  DefaultOptionHandler(const std::string& optName,
-                       const std::string& description = NO_DESCRIPTION,
+  DefaultOptionHandler(const Pref* pref,
+                       const char* description = NO_DESCRIPTION,
                        const std::string& defaultValue = NO_DEFAULT_VALUE,
                        const std::string& possibleValuesString = A2STR::NIL,
                        OptionHandler::ARG_TYPE argType = OptionHandler::REQ_ARG,
@@ -156,13 +133,13 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class CumulativeOptionHandler : public NameMatchOptionHandler {
+class CumulativeOptionHandler : public AbstractOptionHandler {
 private:
   std::string delim_;
   std::string possibleValuesString_;
 public:
-  CumulativeOptionHandler(const std::string& optName,
-                          const std::string& description,
+  CumulativeOptionHandler(const Pref* pref,
+                          const char* description,
                           const std::string& defaultValue,
                           const std::string& delim,
                           const std::string& possibleValuesString = A2STR::NIL,
@@ -174,50 +151,50 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class IndexOutOptionHandler : public NameMatchOptionHandler {
-private:
+class IndexOutOptionHandler : public AbstractOptionHandler {
 public:
-  IndexOutOptionHandler(const std::string& optName,
-                        const std::string& description,
+  IndexOutOptionHandler(const Pref* pref,
+                        const char* description,
                         char shortName = 0);
   virtual ~IndexOutOptionHandler();
   virtual void parseArg(Option& option, const std::string& optarg);
   virtual std::string createPossibleValuesString() const;
 };
 
-class ChecksumOptionHandler : public NameMatchOptionHandler {
-private:
+#ifdef ENABLE_MESSAGE_DIGEST
+class ChecksumOptionHandler : public AbstractOptionHandler {
 public:
-  ChecksumOptionHandler(const std::string& optName,
-                        const std::string& description,
+  ChecksumOptionHandler(const Pref* pref,
+                        const char* description,
                         char shortName = 0);
   virtual ~ChecksumOptionHandler();
   virtual void parseArg(Option& option, const std::string& optarg);
   virtual std::string createPossibleValuesString() const;
 };
+#endif // ENABLE_MESSAGE_DIGEST
 
-class ParameterOptionHandler : public NameMatchOptionHandler {
+class ParameterOptionHandler : public AbstractOptionHandler {
 private:
   std::vector<std::string> validParamValues_;
 public:
-  ParameterOptionHandler(const std::string& optName,
-                         const std::string& description,
+  ParameterOptionHandler(const Pref* pref,
+                         const char* description,
                          const std::string& defaultValue,
                          const std::vector<std::string>& validParamValues,
                          char shortName = 0);
-  ParameterOptionHandler(const std::string& optName,
-                         const std::string& description,
+  ParameterOptionHandler(const Pref* pref,
+                         const char* description,
                          const std::string& defaultValue,
                          const std::string& validParamValue,
                          char shortName = 0);
-  ParameterOptionHandler(const std::string& optName,
-                         const std::string& description,
+  ParameterOptionHandler(const Pref* pref,
+                         const char* description,
                          const std::string& defaultValue,
                          const std::string& validParamValue1,
                          const std::string& validParamValue2,
                          char shortName = 0);
-  ParameterOptionHandler(const std::string& optName,
-                         const std::string& description,
+  ParameterOptionHandler(const Pref* pref,
+                         const char* description,
                          const std::string& defaultValue,
                          const std::string& validParamValue1,
                          const std::string& validParamValue2,
@@ -228,16 +205,16 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class HostPortOptionHandler : public NameMatchOptionHandler {
+class HostPortOptionHandler : public AbstractOptionHandler {
 private:
-  std::string hostOptionName_;
-  std::string portOptionName_;
+  const Pref* hostOptionName_;
+  const Pref* portOptionName_;
 public:
-  HostPortOptionHandler(const std::string& optName,
-                        const std::string& description,
+  HostPortOptionHandler(const Pref* pref,
+                        const char* description,
                         const std::string& defaultValue,
-                        const std::string& hostOptionName,
-                        const std::string& portOptionName,
+                        const Pref* hostOptionName,
+                        const Pref* portOptionName,
                         char shortName = 0);
   virtual ~HostPortOptionHandler();
   virtual void parseArg(Option& option, const std::string& optarg);
@@ -246,33 +223,13 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class HttpProxyUserOptionHandler:public NameMatchOptionHandler {
-public:
-  HttpProxyUserOptionHandler(const std::string& optName,
-                             const std::string& description,
-                             const std::string& defaultValue,
-                             char shortName = 0);
-  virtual void parseArg(Option& option, const std::string& optarg);
-  virtual std::string createPossibleValuesString() const;
-};
-
-class HttpProxyPasswdOptionHandler:public NameMatchOptionHandler {
-public:
-  HttpProxyPasswdOptionHandler(const std::string& optName,
-                               const std::string& description,
-                               const std::string& defaultValue,
-                               char shortName = 0);
-  virtual void parseArg(Option& option, const std::string& optarg);
-  virtual std::string createPossibleValuesString() const;
-};
-
-class HttpProxyOptionHandler : public NameMatchOptionHandler {
+class HttpProxyOptionHandler : public AbstractOptionHandler {
 private:
-  std::string proxyUserPref_;
-  std::string proxyPasswdPref_;
+  const Pref* proxyUserPref_;
+  const Pref* proxyPasswdPref_;
 public:
-  HttpProxyOptionHandler(const std::string& optName,
-                         const std::string& description,
+  HttpProxyOptionHandler(const Pref* pref,
+                         const char* description,
                          const std::string& defaultValue,
                          char shortName = 0);
   virtual ~HttpProxyOptionHandler();
@@ -280,13 +237,13 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class LocalFilePathOptionHandler : public NameMatchOptionHandler {
+class LocalFilePathOptionHandler : public AbstractOptionHandler {
 private:
   bool acceptStdin_;
 public:
   LocalFilePathOptionHandler
-  (const std::string& optName,
-   const std::string& description = NO_DESCRIPTION,
+  (const Pref* pref,
+   const char* description = NO_DESCRIPTION,
    const std::string& defaultValue = NO_DEFAULT_VALUE,
    bool acceptStdin = false,
    char shortName = 0);
@@ -294,11 +251,11 @@ public:
   virtual std::string createPossibleValuesString() const;
 };
 
-class PrioritizePieceOptionHandler:public NameMatchOptionHandler {
+class PrioritizePieceOptionHandler:public AbstractOptionHandler {
 public:
   PrioritizePieceOptionHandler
-  (const std::string& optName,
-   const std::string& description = NO_DESCRIPTION,
+  (const Pref* pref,
+   const char* description = NO_DESCRIPTION,
    const std::string& defaultValue = NO_DEFAULT_VALUE,
    char shortName = 0);
   virtual void parseArg(Option& option, const std::string& optarg);
@@ -319,23 +276,31 @@ public:
   (const SharedHandle<OptionHandler>& depOptHandler,
    const SharedHandle<OptionHandler>& repOptHandler =
    SharedHandle<OptionHandler>());
-  virtual bool canHandle(const std::string& optName);
   virtual void parse(Option& option, const std::string& arg);
   virtual std::string createPossibleValuesString() const;
   virtual bool hasTag(const std::string& tag) const;
   virtual void addTag(const std::string& tag);
   virtual std::string toTagString() const;
-  virtual const std::string& getName() const;
-  virtual const std::string& getDescription() const;
+  virtual const char* getName() const;
+  virtual const char* getDescription() const;
   virtual const std::string& getDefaultValue() const;
   virtual bool isHidden() const;
   virtual void hide();
+  virtual const Pref* getPref() const;
   virtual ARG_TYPE getArgType() const;
   virtual char getShortName() const;
-  virtual int getOptionID() const;
-  virtual void setOptionID(int id);
   virtual bool getEraseAfterParse() const;
   virtual void setEraseAfterParse(bool eraseAfterParse);
+  virtual bool getInitialOption() const;
+  virtual void setInitialOption(bool f);
+  virtual bool getChangeOption() const;
+  virtual void setChangeOption(bool f);
+  virtual bool getChangeOptionForReserved() const;
+  virtual void setChangeOptionForReserved(bool f);
+  virtual bool getChangeGlobalOption() const;
+  virtual void setChangeGlobalOption(bool f);
+  virtual bool getCumulative() const;
+  virtual void setCumulative(bool f);
 };
 
 } // namespace aria2
