@@ -35,8 +35,6 @@
 #include "util.h"
 
 #include <signal.h>
-#include <limits.h>
-#include <stdint.h>
 
 #include <cerrno>
 #include <cassert>
@@ -51,15 +49,12 @@
 
 #include "SimpleRandomizer.h"
 #include "File.h"
-#include "message.h"
 #include "Randomizer.h"
 #include "a2netcompat.h"
-#include "DlAbortEx.h"
 #include "BitfieldMan.h"
 #include "DefaultDiskWriter.h"
 #include "FatalException.h"
 #include "FileEntry.h"
-#include "fmt.h"
 #include "A2STR.h"
 #include "array_fun.h"
 #include "bitfield.h"
@@ -70,6 +65,8 @@
 #include "Option.h"
 #include "DownloadContext.h"
 #include "BufferedFile.h"
+#include "SocketCore.h"
+#include "prefs.h"
 
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigest.h"
@@ -188,22 +185,10 @@ const std::string DEFAULT_STRIP_CHARSET("\r\n\t ");
 
 std::string strip(const std::string& str, const std::string& chars)
 {
-  return stripIter(str.begin(), str.end(), chars);
-}
-
-void divide
-(std::pair<std::string, std::string>& hp, const std::string& src, char delim)
-{
-  std::string::const_iterator first = src.begin();
-  std::string::const_iterator last = src.end();
-  std::string::const_iterator dpos = std::find(first, last, delim);
-  if(dpos == last) {
-    hp.first = strip(src);
-    hp.second = A2STR::NIL;
-  } else {
-    hp.first = stripIter(first, dpos);
-    hp.second = stripIter(dpos+1, last);
-  }
+  std::pair<std::string::const_iterator,
+            std::string::const_iterator> p =
+    stripIter(str.begin(), str.end(), chars);
+  return std::string(p.first, p.second);
 }
 
 std::string itos(int64_t value, bool comma)
@@ -245,34 +230,6 @@ int32_t difftvsec(struct timeval tv1, struct timeval tv2) {
   return tv1.tv_sec-tv2.tv_sec;
 }
 
-bool startsWith(const std::string& target, const std::string& part) {
-  if(target.size() < part.size()) {
-    return false;
-  }
-  if(part.empty()) {
-    return true;
-  }
-  if(target.find(part) == 0) {
-    return true;
-  } else {
-    return false;
-  }
-}
-
-bool endsWith(const std::string& target, const std::string& part) {
-  if(target.size() < part.size()) {
-    return false;
-  }
-  if(part.empty()) {
-    return true;
-  }
-  if(target.rfind(part) == target.size()-part.size()) {
-    return true;
-  } else {
-    return false;
-  }
-}
-
 std::string replace(const std::string& target, const std::string& oldstr, const std::string& newstr) {
   if(target.empty() || oldstr.empty()) {
     return target;
@@ -281,13 +238,12 @@ std::string replace(const std::string& target, const std::string& oldstr, const
   std::string::size_type p = 0;
   std::string::size_type np = target.find(oldstr);
   while(np != std::string::npos) {
-    result += target.substr(p, np-p);
+    result.append(target.begin()+p, target.begin()+np);
     result += newstr;
     p = np+oldstr.size();
     np = target.find(oldstr, p);
   }
-  result += target.substr(p);
-
+  result.append(target.begin()+p, target.end());
   return result;
 }
 
@@ -481,20 +437,21 @@ std::string torrentPercentEncode(const std::string& target)
     (reinterpret_cast<const unsigned char*>(target.c_str()), target.size());
 }
 
-std::string percentDecode(const std::string& target) {
+std::string percentDecode
+(std::string::const_iterator first, std::string::const_iterator last)
+{
   std::string result;
-  for(std::string::const_iterator itr = target.begin(), eoi = target.end();
-      itr != eoi; ++itr) {
-    if(*itr == '%') {
-      if(itr+1 != target.end() && itr+2 != target.end() &&
-         isHexDigit(*(itr+1)) && isHexDigit(*(itr+2))) {
-        result += parseInt(std::string(itr+1, itr+3), 16);
-        itr += 2;
+  for(; first != last; ++first) {
+    if(*first == '%') {
+      if(first+1 != last && first+2 != last &&
+         isHexDigit(*(first+1)) && isHexDigit(*(first+2))) {
+        result += parseInt(std::string(first+1, first+3), 16);
+        first += 2;
       } else {
-        result += *itr;
+        result += *first;
       }
     } else {
-      result += *itr;
+      result += *first;
     }
   }
   return result;
@@ -529,10 +486,8 @@ std::string toHex(const std::string& src)
   return toHex(reinterpret_cast<const unsigned char*>(src.c_str()), src.size());
 }
 
-namespace {
 unsigned int hexCharToUInt(unsigned char ch)
 {
-
   if('a' <= ch && ch <= 'f') {
     ch -= 'a';
     ch += 10;
@@ -546,25 +501,6 @@ unsigned int hexCharToUInt(unsigned char ch)
   }
   return ch;
 }
-} // namespace
-
-std::string fromHex(const std::string& src)
-{
-  std::string dest;
-  if(src.size()%2) {
-    return dest;
-  }
-  for(size_t i = 0, eoi = src.size(); i < eoi; i += 2) {
-    unsigned char high = hexCharToUInt(src[i]);
-    unsigned char low = hexCharToUInt(src[i+1]);
-    if(high == 255 || low == 255) {
-      dest.clear();
-      return dest;
-    }
-    dest += (high*16+low);
-  }
-  return dest;
-}
 
 FILE* openFile(const std::string& filename, const std::string& mode) {
   FILE* file = fopen(filename.c_str(), mode.c_str());
@@ -585,26 +521,19 @@ bool isPowerOf(int num, int base) {
 }
 
 std::string secfmt(time_t sec) {
+  time_t tsec = sec;
   std::string str;
   if(sec >= 3600) {
-    str = itos(sec/3600);
-    str += "h";
+    str = fmt("%lldh", static_cast<long long int>(sec/3600));
     sec %= 3600;
   }
   if(sec >= 60) {
-    int min = sec/60;
-    if(min < 10) {
-      str += "0";
-    }
-    str += itos(min);
-    str += "m";
+    str += fmt("%dm", static_cast<int>(sec/60));
     sec %= 60;
   }
-  if(sec < 10) {
-    str += "0";
+  if(sec || tsec == 0) {
+    str += fmt("%ds", static_cast<int>(sec));
   }
-  str += itos(sec);
-  str += "s";
   return str;
 }
 
@@ -617,159 +546,129 @@ int getNum(const char* buf, int offset, size_t length) {
   return x;
 }
 
-int32_t parseInt(const std::string& s, int32_t base)
+namespace {
+template<typename T, typename F>
+bool parseLong(T& res, F f, const std::string& s, int base)
 {
-  int64_t v = parseLLInt(s, base);
-  if(v < INT32_MIN || INT32_MAX < v) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          s.c_str()));
+  if(s.empty()) {
+    return false;
+  }
+  char* endptr;
+  errno = 0;
+  res = f(s.c_str(), &endptr, base);
+  if(errno == ERANGE) {
+    return false;
+  }
+  if(*endptr != '\0') {
+    for(const char* i = endptr, *eoi = s.c_str()+s.size(); i < eoi; ++i) {
+      if(!isspace(*i)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+} // namespace
+
+bool parseIntNoThrow(int32_t& res, const std::string& s, int base)
+{
+  long int t;
+  if(parseLong(t, strtol, s, base) &&
+     t >= std::numeric_limits<int32_t>::min() &&
+     t <= std::numeric_limits<int32_t>::max()) {
+    res = t;
+    return true;
+  } else {
+    return false;
   }
-  return v;
 }
 
-bool parseIntNoThrow(int32_t& result, const std::string& s, int base)
+int32_t parseInt(const std::string& s, int base)
 {
-  // Without trim, strtol("  -1  ",..) emits error.
-  std::string trimed = strip(s);
-  if(trimed.empty()) {
+  int32_t res;
+  if(parseIntNoThrow(res, s, base)) {
+    return res;
+  } else {
+    throw DL_ABORT_EX
+      (fmt("Failed to convert string into 32bit signed integer. '%s'",
+           s.c_str()));
+  }
+}
+
+bool parseUIntNoThrow(uint32_t& res, const std::string& s, int base)
+{
+  long int t;
+  if(parseLong(t, strtol, s, base) &&
+     t >= 0 &&
+     t <= std::numeric_limits<int32_t>::max()) {
+    res = t;
+    return true;
+  } else {
     return false;
   }
-  char* stop;
-  errno = 0;
-  long int v = strtol(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    return false;
-  } else if(((v == LONG_MAX || v == LONG_MIN) && (errno == ERANGE)) ||
-            v < INT32_MIN || INT32_MAX < v) {
-    return false;
-  }
-  result = v;
-  return true;
 }
 
 uint32_t parseUInt(const std::string& s, int base)
 {
-  uint64_t v = parseULLInt(s, base);
-  if(UINT32_MAX < v) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          s.c_str()));
+  uint32_t res;
+  if(parseUIntNoThrow(res, s, base)) {
+    return res;
+  } else {
+    throw DL_ABORT_EX
+      (fmt("Failed to convert string into 32bit unsigned integer. '%s'",
+           s.c_str()));
   }
-  return v;
 }
 
-bool parseUIntNoThrow(uint32_t& result, const std::string& s, int base)
+bool parseLLIntNoThrow(int64_t& res, const std::string& s, int base)
 {
-  // Without trim, strtol("  -1  ",..) emits error.
-  std::string trimed = strip(s);
-  if(trimed.empty()) {
+  long long int t;
+  if(parseLong(t, strtoll, s, base) &&
+     t >= std::numeric_limits<int64_t>::min() &&
+     t <= std::numeric_limits<int64_t>::max()) {
+    res = t;
+    return true;
+  } else {
     return false;
   }
-  // We don't allow negative number.
-  if(trimed[0] == '-') {
-    return false;
-  }
-  char* stop;
-  errno = 0;
-  unsigned long int v = strtoul(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    return false;
-  } else if(((v == ULONG_MAX) && (errno == ERANGE)) || (v > UINT32_MAX)) {
-    return false;
-  }
-  result = v;
-  return true;
 }
 
-int64_t parseLLInt(const std::string& s, int32_t base)
+int64_t parseLLInt(const std::string& s, int base)
 {
-  std::string trimed = strip(s);
-  if(trimed.empty()) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          "empty string"));
+  int64_t res;
+  if(parseLLIntNoThrow(res, s, base)) {
+    return res;
+  } else {
+    throw DL_ABORT_EX
+      (fmt("Failed to convert string into 64bit signed integer. '%s'",
+           s.c_str()));
   }
-  char* stop;
-  errno = 0;
-  int64_t v = strtoll(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          trimed.c_str()));
-  } else if(((v == INT64_MIN) || (v == INT64_MAX)) && (errno == ERANGE)) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          trimed.c_str()));
-  }
-  return v;
 }
 
-bool parseLLIntNoThrow(int64_t& result, const std::string& s, int base)
+void parseIntSegments(SegList<int>& sgl, const std::string& src)
 {
-  // Without trim, strtol("  -1  ",..) emits error.
-  std::string trimed = strip(s);
-  if(trimed.empty()) {
-    return false;
-  }
-  char* stop;
-  errno = 0;
-  int64_t v = strtoll(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    return false;
-  } else if(((v == INT64_MIN) || (v == INT64_MAX)) && (errno == ERANGE)) {
-    return false;
-  }
-  result = v;
-  return true;
-}
-
-uint64_t parseULLInt(const std::string& s, int base)
-{
-  std::string trimed = strip(s);
-  if(trimed.empty()) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          "empty string"));
-  }
-  // We don't allow negative number.
-  if(trimed[0] == '-') {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          trimed.c_str()));
-  }
-  char* stop;
-  errno = 0;
-  uint64_t v = strtoull(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          trimed.c_str()));
-  } else if((v == ULLONG_MAX) && (errno == ERANGE)) {
-    throw DL_ABORT_EX(fmt(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-                          trimed.c_str()));
-  }
-  return v;
-}
-
-IntSequence parseIntRange(const std::string& src)
-{
-  IntSequence::Values values;
-  std::string temp = src;
-  while(temp.size()) {
-    std::pair<std::string, std::string> p;
-    divide(p, temp, ',');
-    temp = p.second;
-    if(p.first.empty()) {
+  for(std::string::const_iterator i = src.begin(), eoi = src.end(); i != eoi;) {
+    std::string::const_iterator j = std::find(i, eoi, ',');
+    if(j == i) {
+      ++i;
       continue;
     }
-    if(p.first.find("-") == std::string::npos) {
-      int32_t v = parseInt(p.first.c_str());
-      values.push_back(IntSequence::Value(v, v+1));
+    std::string::const_iterator p = std::find(i, j, '-');
+    if(p == j) {
+      int a = parseInt(std::string(i, j));
+      sgl.add(a, a+1);
+    } else if(p == i || p+1 == j) {
+      throw DL_ABORT_EX(fmt(MSG_INCOMPLETE_RANGE, std::string(i, j).c_str()));
     } else {
-      std::pair<std::string, std::string> vp;
-      divide(vp, p.first.c_str(), '-');
-      if(vp.first.empty() || vp.second.empty()) {
-        throw DL_ABORT_EX(fmt(MSG_INCOMPLETE_RANGE, p.first.c_str()));
-      }
-      int32_t v1 = parseInt(vp.first.c_str());
-      int32_t v2 = parseInt(vp.second.c_str());
-      values.push_back(IntSequence::Value(v1, v2+1));
-    } 
+      int a = parseInt(std::string(i, p));
+      int b = parseInt(std::string(p+1, j));
+      sgl.add(a, b+1);
+    }
+    if(j == eoi) {
+      break;
+    }
+    i = j+1;
   }
-  return values;
 }
 
 namespace {
@@ -777,7 +676,7 @@ void computeHeadPieces
 (std::vector<size_t>& indexes,
  const std::vector<SharedHandle<FileEntry> >& fileEntries,
  size_t pieceLength,
- uint64_t head)
+ off_t head)
 {
   if(head == 0) {
     return;
@@ -802,7 +701,7 @@ void computeTailPieces
 (std::vector<size_t>& indexes,
  const std::vector<SharedHandle<FileEntry> >& fileEntries,
  size_t pieceLength,
- uint64_t tail)
+ off_t tail)
 {
   if(tail == 0) {
     return;
@@ -812,7 +711,7 @@ void computeTailPieces
     if((*fi)->getLength() == 0) {
       continue;
     }
-    uint64_t endOffset = (*fi)->getLastOffset();
+    int64_t endOffset = (*fi)->getLastOffset();
     size_t fromIndex =
       (endOffset-1-(std::min(tail, (*fi)->getLength())-1))/pieceLength;
     for(size_t index = fromIndex; index <= (endOffset-1)/pieceLength;
@@ -827,29 +726,28 @@ void parsePrioritizePieceRange
 (std::vector<size_t>& result, const std::string& src,
  const std::vector<SharedHandle<FileEntry> >& fileEntries,
  size_t pieceLength,
- uint64_t defaultSize)
+ off_t defaultSize)
 {
   std::vector<size_t> indexes;
-  std::vector<std::string> parts;
-  split(src, std::back_inserter(parts), ",", true);
-  for(std::vector<std::string>::const_iterator i = parts.begin(),
+  std::vector<Scip> parts;
+  splitIter(src.begin(), src.end(), std::back_inserter(parts), ',', true);
+  for(std::vector<Scip>::const_iterator i = parts.begin(),
         eoi = parts.end(); i != eoi; ++i) {
-    if((*i) == "head") {
+    if(util::streq((*i).first, (*i).second, "head")) {
       computeHeadPieces(indexes, fileEntries, pieceLength, defaultSize);
-    } else if(util::startsWith(*i, "head=")) {
-      std::string sizestr = std::string((*i).begin()+(*i).find("=")+1,
-                                        (*i).end());
+    } else if(util::startsWith((*i).first, (*i).second, "head=")) {
+      std::string sizestr((*i).first+5, (*i).second);
       computeHeadPieces(indexes, fileEntries, pieceLength,
                         std::max((int64_t)0, getRealSize(sizestr)));
-    } else if((*i) == "tail") {
+    } else if(util::streq((*i).first, (*i).second, "tail")) {
       computeTailPieces(indexes, fileEntries, pieceLength, defaultSize);
-    } else if(util::startsWith(*i, "tail=")) {
-      std::string sizestr = std::string((*i).begin()+(*i).find("=")+1,
-                                        (*i).end());
+    } else if(util::startsWith((*i).first, (*i).second, "tail=")) {
+      std::string sizestr((*i).first+5, (*i).second);
       computeTailPieces(indexes, fileEntries, pieceLength,
                         std::max((int64_t)0, getRealSize(sizestr)));
     } else {
-      throw DL_ABORT_EX(fmt("Unrecognized token %s", (*i).c_str()));
+      throw DL_ABORT_EX(fmt("Unrecognized token %s",
+                            std::string((*i).first, (*i).second).c_str()));
     }
   }
   std::sort(indexes.begin(), indexes.end());
@@ -894,7 +792,7 @@ void parseParam(OutputIterator out, const std::string& header)
     std::string param;
     if(paramLast == eoi || *paramLast == ';') {
       // No value, parmname only
-      param = std::string(paramFirst, paramLast);
+      param.assign(paramFirst, paramLast);
     } else {
       for(; paramLast != eoi && *paramLast != '"' && *paramLast != ';';
           ++paramLast);
@@ -905,10 +803,10 @@ void parseParam(OutputIterator out, const std::string& header)
         if(paramLast != eoi) {
           ++paramLast;
         }
-        param = std::string(paramFirst, paramLast);
+        param.assign(paramFirst, paramLast);
         for(; paramLast != eoi && *paramLast != ';'; ++paramLast);
       } else {
-        param = std::string(paramFirst, paramLast);
+        param.assign(paramFirst, paramLast);
       }
     }
     param = strip(param);
@@ -924,17 +822,18 @@ void parseParam(OutputIterator out, const std::string& header)
 
 std::string getContentDispositionFilename(const std::string& header)
 {
+  static const char A2_KEYNAME[] = "filename";
   std::string filename;
   std::vector<std::string> params;
   parseParam(std::back_inserter(params), header);
   for(std::vector<std::string>::const_iterator i = params.begin(),
         eoi = params.end(); i != eoi; ++i) {
     const std::string& param = *i;
-    static const std::string keyName = "filename";
-    if(!startsWith(toLower(param), keyName) || param.size() == keyName.size()) {
+    if(!istartsWith(param, A2_KEYNAME) ||
+       param.size() == sizeof(A2_KEYNAME)-1) {
       continue;
     }
-    std::string::const_iterator markeritr = param.begin()+keyName.size();
+    std::string::const_iterator markeritr = param.begin()+sizeof(A2_KEYNAME)-1;
     if(*markeritr == '*') {
       // See RFC2231 Section4 and draft-reschke-rfc2231-in-http.
       // Please note that this function doesn't do charset conversion
@@ -945,18 +844,15 @@ std::string getContentDispositionFilename(const std::string& header)
       if(markeritr == param.end() || *markeritr != '=') {
         continue;
       }
-      std::pair<std::string, std::string> paramPair;
-      divide(paramPair, param, '=');
-      std::string value = paramPair.second;
-      std::vector<std::string> extValues;
-      split(value, std::back_inserter(extValues), "'", false, true);
+      std::vector<Scip> extValues;
+      splitIter(markeritr+1, param.end(), std::back_inserter(extValues),
+                '\'', true, true);
       if(extValues.size() != 3) {
         continue;
       }
       bool bad = false;
-      const std::string& charset = extValues[0];
-      for(std::string::const_iterator j = charset.begin(), eoi = charset.end();
-          j != eoi; ++j) {
+      for(std::string::const_iterator j = extValues[0].first,
+            eoj = extValues[0].second; j != eoj; ++j) {
         // Since we first split parameter by ', we can safely assume
         // that ' is not included in charset.
         if(!inRFC2978MIMECharset(*j)) {
@@ -968,12 +864,11 @@ std::string getContentDispositionFilename(const std::string& header)
         continue;
       }
       bad = false;
-      value = extValues[2];
-      for(std::string::const_iterator j = value.begin(), eoi = value.end();
-          j != eoi; ++j){
+      for(std::string::const_iterator j = extValues[2].first,
+            eoj = extValues[2].second; j != eoj; ++j){
         if(*j == '%') {
-          if(j+1 != value.end() && isHexDigit(*(j+1)) &&
-             j+2 != value.end() && isHexDigit(*(j+2))) {
+          if(j+1 != eoj && isHexDigit(*(j+1)) &&
+             j+2 != eoj && isHexDigit(*(j+2))) {
             j += 2;
           } else {
             bad = true;
@@ -989,12 +884,12 @@ std::string getContentDispositionFilename(const std::string& header)
       if(bad) {
         continue;
       }
-      value = percentDecode(value);
-      if(toLower(extValues[0]) == "iso-8859-1") {
+      std::string value =
+        percentDecode(extValues[2].first, extValues[2].second);
+      if(util::strieq(extValues[0].first, extValues[0].second, "iso-8859-1")) {
         value = iso8859ToUtf8(value);
       }
-      if(!detectDirTraversal(value) &&
-         value.find(A2STR::SLASH_C) == std::string::npos) {
+      if(!detectDirTraversal(value) && value.find("/") == std::string::npos) {
         filename = value;
       }
       if(!filename.empty()) {
@@ -1002,17 +897,17 @@ std::string getContentDispositionFilename(const std::string& header)
       }
     } else {
       for(; markeritr != param.end() && *markeritr == ' '; ++markeritr);
-      if(markeritr == param.end() || *markeritr != '=') {
+      if(markeritr == param.end() || markeritr+1 == param.end() ||
+         *markeritr != '=') {
         continue;
       }
-      std::pair<std::string, std::string> paramPair;
-      divide(paramPair, param, '=');
-      std::string value = paramPair.second;
-      if(value.empty()) {
+      Scip p = stripIter(markeritr+1, param.end());
+      if(p.first == p.second) {
         continue;
       }
+      std::string value(p.first, p.second);
       std::string::iterator filenameLast;
-      if(*value.begin() == '\'' || *value.begin() == '"') {
+      if(value[0] == '\'' || value[0] == '"') {
         char qc = *value.begin();
         for(filenameLast = value.begin()+1;
             filenameLast != value.end() && *filenameLast != qc;
@@ -1021,11 +916,10 @@ std::string getContentDispositionFilename(const std::string& header)
         filenameLast = value.end();
       }
       static const std::string TRIMMED("\r\n\t '\"");
-      value = percentDecode(std::string(value.begin(), filenameLast));
+      value = percentDecode(value.begin(), filenameLast);
       value = strip(value, TRIMMED);
       value.erase(std::remove(value.begin(), value.end(), '\\'), value.end());
-      if(!detectDirTraversal(value) &&
-         value.find(A2STR::SLASH_C) == std::string::npos) {
+      if(!detectDirTraversal(value) && value.find("/") == std::string::npos) {
         filename = value;
       }
       // continue because there is a chance we can find filename*=...
@@ -1035,11 +929,12 @@ std::string getContentDispositionFilename(const std::string& header)
 }
 
 std::string randomAlpha(size_t length, const RandomizerHandle& randomizer) {
-  static const char *random_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
+  static const char randomChars[] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
   std::string str;
   for(size_t i = 0; i < length; ++i) {
-    size_t index = randomizer->getRandomNumber(strlen(random_chars));
-    str += random_chars[index];
+    size_t index = randomizer->getRandomNumber(sizeof(randomChars)-1);
+    str += randomChars[index];
   }
   return str;
 }
@@ -1131,7 +1026,7 @@ int64_t getRealSize(const std::string& sizeWithUnit)
     } else if(sizeWithUnit[p] == 'M') {
       mult = 1024*1024;
     }
-    size = sizeWithUnit.substr(0, p);
+    size.assign(sizeWithUnit.begin(), sizeWithUnit.begin()+p);
   }
   int64_t v = parseLLInt(size);
 
@@ -1149,20 +1044,16 @@ std::string abbrevSize(int64_t size)
   if(size < 1024) {
     return itos(size, true);
   }
-  char units[] = { 'K', 'M' };
-  size_t numUnit = sizeof(units)/sizeof(char);
+  static const char units[] = { 'K', 'M' };
   size_t i = 0;
   int r = size&0x3ffu;
   size >>= 10;
-  for(; i < numUnit-1 && size >= 1024; ++i) {
+  for(; i < sizeof(units)-1 && size >= 1024; ++i) {
     r = size&0x3ffu;
     size >>= 10;
   }
   std::string result = itos(size, true);
-  result += A2STR::DOT_C;
-  result += itos(r*10/1024);
-  result += units[i];
-  result += "i";
+  result += fmt(".%d%ci", r*10/1024, units[i]);
   return result;
 }
 
@@ -1213,46 +1104,6 @@ void usleep(long microseconds) {
 #endif
 }
 
-bool isNumber(const std::string& what)
-{
-  if(what.empty()) {
-    return false;
-  }
-  for(std::string::const_iterator i = what.begin(), eoi = what.end();
-      i != eoi; ++i) {
-    if(!isDigit(*i)) {
-      return false;
-    }
-  }
-  return true;
-}
-
-bool isLowercase(const std::string& what)
-{
-  if(what.empty()) {
-    return false;
-  }
-  for(uint32_t i = 0, eoi = what.size(); i < eoi; ++i) {
-    if(!('a' <= what[i] && what[i] <= 'z')) {
-      return false;
-    }
-  }
-  return true;
-}
-
-bool isUppercase(const std::string& what)
-{
-  if(what.empty()) {
-    return false;
-  }
-  for(uint32_t i = 0, eoi = what.size(); i < eoi; ++i) {
-    if(!('A' <= what[i] && what[i] <= 'Z')) {
-      return false;
-    }
-  }
-  return true;
-}
-
 unsigned int alphaToNum(const std::string& alphabets)
 {
   if(alphabets.empty()) {
@@ -1294,7 +1145,7 @@ void convertBitfield(BitfieldMan* dest, const BitfieldMan* src)
 {
   size_t numBlock = dest->countBlock();
   for(size_t index = 0; index < numBlock; ++index) {
-    if(src->isBitSetOffsetRange((uint64_t)index*dest->getBlockLength(),
+    if(src->isBitSetOffsetRange((off_t)index*dest->getBlockLength(),
                                 dest->getBlockLength())) {
       dest->setBit(index);
     }
@@ -1370,36 +1221,69 @@ std::string htmlEscape(const std::string& src)
   return dest;
 }
 
-std::map<size_t, std::string>::value_type
+std::pair<size_t, std::string>
 parseIndexPath(const std::string& line)
 {
-  std::pair<std::string, std::string> p;
-  divide(p, line, '=');
-  size_t index = parseUInt(p.first);
-  if(p.second.empty()) {
+  std::pair<Scip, Scip> p;
+  divide(p, line.begin(), line.end(), '=');
+  size_t index = parseUInt(std::string(p.first.first, p.first.second));
+  if(p.second.first == p.second.second) {
     throw DL_ABORT_EX(fmt("Path with index=%u is empty.",
                           static_cast<unsigned int>(index)));
   }
-  return std::map<size_t, std::string>::value_type(index, p.second);
+  return std::make_pair(index, std::string(p.second.first, p.second.second));
 }
 
-std::map<size_t, std::string> createIndexPathMap(std::istream& i)
+std::vector<std::pair<size_t, std::string> > createIndexPaths(std::istream& i)
 {
-  std::map<size_t, std::string> indexPathMap;
+  std::vector<std::pair<size_t, std::string> > indexPaths;
   std::string line;
   while(getline(i, line)) {
-    indexPathMap.insert(indexPathMap.begin(), parseIndexPath(line));
+    indexPaths.push_back(parseIndexPath(line));
   }
-  return indexPathMap;
+  return indexPaths;
 }
 
-void generateRandomData(unsigned char* data, size_t length)
+namespace {
+void generateRandomDataRandom(unsigned char* data, size_t length)
 {
   const SharedHandle<SimpleRandomizer>& rd = SimpleRandomizer::getInstance();
   for(size_t i = 0; i < length; ++i) {
     data[i] = static_cast<unsigned long>(rd->getRandomNumber(256));
   }
 }
+} // namespace
+
+namespace {
+void generateRandomDataUrandom
+(unsigned char* data, size_t length, std::ifstream& devUrand)
+{
+  devUrand.read(reinterpret_cast<char*>(data), length);
+}
+} // namespace
+
+void generateRandomData(unsigned char* data, size_t length)
+{
+#ifdef __MINGW32__
+  generateRandomDataRandom(data, length);
+#else // !__MINGW32__
+  static int method = -1;
+  static std::ifstream devUrand;
+  if(method == 0) {
+    generateRandomDataUrandom(data, length, devUrand);
+  } else if(method == 1) {
+    generateRandomDataRandom(data, length);
+  } else {
+    devUrand.open("/dev/urandom");
+    if(devUrand) {
+      method = 0;
+    } else {
+      method = 1;
+    }
+    generateRandomData(data, length);
+  }
+#endif // !__MINGW32__
+}
 
 bool saveAs
 (const std::string& filename, const std::string& data, bool overwrite)
@@ -1407,7 +1291,8 @@ bool saveAs
   if(!overwrite && File(filename).exists()) {
     return false;
   }
-  std::string tempFilename = strconcat(filename, "__temp");
+  std::string tempFilename = filename;
+  tempFilename += "__temp";
   {
     BufferedFile fp(tempFilename, BufferedFile::WRITE);
     if(!fp) {
@@ -1427,11 +1312,16 @@ std::string applyDir(const std::string& dir, const std::string& relPath)
 {
   std::string s;
   if(dir.empty()) {
-    s = strconcat(A2STR::DOT_C, A2STR::SLASH_C, relPath);
-  } else if(dir == A2STR::SLASH_C) {
-    s = strconcat(A2STR::SLASH_C, relPath);
+    s = "./";
+    s += relPath;
   } else {
-    s = strconcat(dir, A2STR::SLASH_C, relPath);
+    s = dir;
+    if(dir == "/") {
+      s += relPath;
+    } else {
+      s += "/";
+      s += relPath;
+    }
   }
 #ifdef __MINGW32__
   for(std::string::iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
@@ -1474,7 +1364,9 @@ bool inPrivateAddress(const std::string& ipv4addr)
   }
   if(util::startsWith(ipv4addr, "172.")) {
     for(int i = 16; i <= 31; ++i) {
-      if(util::startsWith(ipv4addr, "172."+util::itos(i)+".")) {
+      std::string t(fmt("%d.", i));
+      if(util::startsWith(ipv4addr.begin()+4, ipv4addr.end(),
+                          t.begin(), t.end())) {
         return true;
       }
     }
@@ -1484,31 +1376,21 @@ bool inPrivateAddress(const std::string& ipv4addr)
 
 bool detectDirTraversal(const std::string& s)
 {
+  if(s.empty()) {
+    return false;
+  }
   for(std::string::const_iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
     unsigned char c = *i;
     if(in(c, 0x00u, 0x1fu) || c == 0x7fu) {
       return true;
     }
   }
-
-  static std::string A2_DS = "./";
-  static std::string A2_DDS = "../";
-  static std::string A2_SD = "/.";
-  static std::string A2_SDD = "/..";
-  static std::string A2_SDDS = "/../";
-  static std::string A2_SDS = "/./";
-  static std::string A2_DD = "..";
-
-  return s == A2STR::DOT_C ||
-    s == A2_DD ||
-    util::startsWith(s, A2STR::SLASH_C) ||
-    util::startsWith(s, A2_DS) ||
-    util::startsWith(s, A2_DDS) ||
-    s.find(A2_SDDS) != std::string::npos ||
-    s.find(A2_SDS) != std::string::npos ||
-    util::endsWith(s, A2STR::SLASH_C) ||
-    util::endsWith(s, A2_SD) ||
-    util::endsWith(s, A2_SDD);
+  return s == "." || s == ".." || s[0] == '/' ||
+    util::startsWith(s, "./") || util::startsWith(s, "../") ||
+    s.find("/../") != std::string::npos ||
+    s.find("/./") != std::string::npos ||
+    s[s.size()-1] == '/' ||
+    util::endsWith(s, "/.") || util::endsWith(s, "/..");
 }
 
 std::string escapePath(const std::string& s)
@@ -1536,37 +1418,30 @@ std::string escapePath(const std::string& s)
   return d;
 }
 
-bool getCidrPrefix(struct in_addr& in, const std::string& ip, int bits)
+bool inSameCidrBlock
+(const std::string& ip1, const std::string& ip2, size_t bits)
 {
-  struct in_addr t;
-  if(inet_aton(ip.c_str(), &t) == 0) {
+  unsigned char s1[16], s2[16];
+  size_t len1, len2;
+  if((len1 = net::getBinAddr(s1, ip1)) == 0 ||
+     (len2 = net::getBinAddr(s2, ip2)) == 0 ||
+     len1 != len2) {
     return false;
   }
-  int lastindex = bits/8;
-  if(lastindex < 4) {
-    char* p = reinterpret_cast<char*>(&t.s_addr);
-    const char* last = p+4;
-    p += lastindex;    
-    if(bits%8 != 0) {
-      *p &= bitfield::lastByteMask(bits);
-      ++p;
-    }
-    for(; p != last; ++p) {
-      *p &= 0;
+  if(bits == 0) {
+    return true;
+  }
+  if(bits > 8*len1) {
+    bits = 8*len1;
+  }
+  int last = (bits-1)/8;
+  for(int i = 0; i < last; ++i) {
+    if(s1[i] != s2[i]) {
+      return false;
     }
   }
-  in = t;
-  return true;
-}
-
-bool inSameCidrBlock(const std::string& ip1, const std::string& ip2, int bits)
-{
-  struct in_addr in1;
-  struct in_addr in2;
-  if(!getCidrPrefix(in1, ip1, bits) || !getCidrPrefix(in2, ip2, bits)) {
-    return false;
-  }
-  return in1.s_addr == in2.s_addr;
+  unsigned char mask = bitfield::lastByteMask(bits);
+  return (s1[last] & mask) == (s2[last] & mask);
 }
 
 void removeMetalinkContentTypes(const SharedHandle<RequestGroup>& group)
@@ -1619,16 +1494,14 @@ void executeHook
 
   memset(&si, 0, sizeof (si));
   si.cb = sizeof(STARTUPINFO);
-
   memset(&pi, 0, sizeof (pi));
-
-  bool batch = util::endsWith(util::toLower(command), ".bat");
+  bool batch = util::iendsWith(command, ".bat");
   std::string cmdline;
   std::string cmdexe;
   if(batch) {
     const char* p = getenv("windir");
     if(p) {
-      cmdexe = std::string(p);
+      cmdexe = p;
       cmdexe += "\\system32\\cmd.exe";
     } else {
       A2_LOG_INFO("Failed to get windir environment variable."
@@ -1638,8 +1511,16 @@ void executeHook
     }
     cmdline += "/C \"";
   }
-  strappend(cmdline, "\"", command, "\"");
-  strappend(cmdline, " ", gidStr, " ", numFilesStr, " \"", firstFilename, "\"");
+  cmdline += "\"";
+  cmdline += command;
+  cmdline += "\"";
+  cmdline += " ";
+  cmdline += gidStr;
+  cmdline += " ";
+  cmdline += numFilesStr;
+  cmdline += " \"";
+  cmdline += firstFilename;
+  cmdline += "\"";
   if(batch) {
     cmdline += "\"";
   }
@@ -1666,15 +1547,16 @@ void executeHook
 
 void executeHookByOptName
 (const SharedHandle<RequestGroup>& group, const Option* option,
- const std::string& opt)
+ const Pref* pref)
 {
-  executeHookByOptName(group.get(), option, opt);
+  executeHookByOptName(group.get(), option, pref);
 }
 
 void executeHookByOptName
-(const RequestGroup* group, const Option* option, const std::string& opt)
+(const RequestGroup* group, const Option* option, const Pref* pref)
 {
-  if(!option->blank(opt)) {
+  const std::string& cmd = option->get(pref);
+  if(!cmd.empty()) {
     const SharedHandle<DownloadContext> dctx = group->getDownloadContext();
     std::string firstFilename;
     size_t numFiles = 0;
@@ -1685,7 +1567,7 @@ void executeHookByOptName
       }
       numFiles = dctx->countRequestedFileEntry();
     }
-    executeHook(option->get(opt), group->getGID(), numFiles, firstFilename);
+    executeHook(cmd, group->getGID(), numFiles, firstFilename);
   }
 }
 
@@ -1718,6 +1600,52 @@ std::string safeStrerror(int errNum)
   return makeString(strerror(errNum));
 }
 
+bool noProxyDomainMatch
+(const std::string& hostname,
+ const std::string& domain)
+{
+  if(!domain.empty() && domain[0] == '.' && !util::isNumericHost(hostname)) {
+    return util::endsWith(hostname, domain);
+  } else {
+    return hostname == domain;
+  }
+}
+
+bool startsWith(const std::string& a, const char* b)
+{
+  return startsWith(a.begin(), a.end(), b);
+}
+
+bool startsWith(const std::string& a, const std::string& b)
+{
+  return startsWith(a.begin(), a.end(), b.begin(), b.end());
+}
+
+bool istartsWith(const std::string& a, const char* b)
+{
+  return istartsWith(a.begin(), a.end(), b);
+}
+
+bool endsWith(const std::string& a, const char* b)
+{
+  return endsWith(a.begin(), a.end(), b, b+strlen(b));
+}
+
+bool endsWith(const std::string& a, const std::string& b)
+{
+  return endsWith(a.begin(), a.end(), b.begin(), b.end());
+}
+
+bool iendsWith(const std::string& a, const char* b)
+{
+  return iendsWith(a.begin(), a.end(), b, b+strlen(b));
+}
+
+bool iendsWith(const std::string& a, const std::string& b)
+{
+  return iendsWith(a.begin(), a.end(), b.begin(), b.end());
+}
+
 } // namespace util
 
 } // namespace aria2
