@@ -38,8 +38,11 @@
 #include "common.h"
 
 #include <sys/time.h>
+#include <limits.h>
+#include <stdint.h>
 
 #include <cstdio>
+#include <cstring>
 #include <string>
 #include <utility>
 #include <iosfwd>
@@ -51,10 +54,14 @@
 #include <vector>
 
 #include "SharedHandle.h"
-#include "IntSequence.h"
 #include "a2time.h"
 #include "a2netcompat.h"
 #include "a2functional.h"
+#include "SegList.h"
+#include "a2iterator.h"
+#include "message.h"
+#include "DlAbortEx.h"
+#include "fmt.h"
 
 namespace aria2 {
 
@@ -64,6 +71,7 @@ class BinaryStream;
 class FileEntry;
 class RequestGroup;
 class Option;
+class Pref;
 
 #define STRTOLL(X) strtoll(X, reinterpret_cast<char**>(0), 10)
 #define STRTOULL(X) strtoull(X, reinterpret_cast<char**>(0), 10)
@@ -86,10 +94,40 @@ inline uint64_t ntoh64(uint64_t x) { return byteswap64(x); }
 inline uint64_t hton64(uint64_t x) { return byteswap64(x); }
 #endif // !WORDS_BIGENDIAN
 
+#ifdef __MINGW32__
+std::wstring utf8ToWChar(const std::string& src);
+
+std::wstring utf8ToWChar(const char* str);
+
+std::string utf8ToNative(const std::string& src);
+
+std::string wCharToUtf8(const std::wstring& wsrc);
+
+std::string nativeToUtf8(const std::string& src);
+#else // !__MINGW32__
+# define utf8ToWChar(src) src
+# define utf8ToNative(src) src
+#endif // !__MINGW32__
+
 namespace util {
 
+template<typename InputIterator>
 void divide
-(std::pair<std::string, std::string>& hp, const std::string& src, char delim);
+(std::pair<std::pair<InputIterator, InputIterator>,
+           std::pair<InputIterator, InputIterator> >& hp,
+ InputIterator first,
+ InputIterator last,
+ char delim)
+{
+  InputIterator dpos = std::find(first, last, delim);
+  if(dpos == last) {
+    hp.first = stripIter(first, last);
+    hp.second.first = hp.second.second;
+  } else {
+    hp.first = stripIter(first, dpos);
+    hp.second = stripIter(dpos+1, last);
+  }
+}
 
 template<typename T>
 std::string uitos(T value, bool comma = false)
@@ -99,12 +137,12 @@ std::string uitos(T value, bool comma = false)
     str = "0";
     return str;
   }
-  unsigned int count = 0;
+  int count = 0;
   while(value) {
     ++count;
     char digit = value%10+'0';
     if(comma && count > 3 && count%3 == 1) {
-      str += ',';
+      str += ",";
     }
     str += digit;
     value /= 10;
@@ -126,44 +164,65 @@ int32_t difftvsec(struct timeval tv1, struct timeval tv2);
 extern const std::string DEFAULT_STRIP_CHARSET;
 
 template<typename InputIterator>
-std::string stripIter
+std::pair<InputIterator, InputIterator> stripIter
 (InputIterator first, InputIterator last,
  const std::string& chars = DEFAULT_STRIP_CHARSET)
 {
-  if(std::distance(first, last) == 0) {
-    return A2STR::NIL;
-  }
   for(; first != last &&
         std::find(chars.begin(), chars.end(), *first) != chars.end(); ++first);
   if(first == last) {
-    return A2STR::NIL;
+    return std::make_pair(first, last);
   }
   InputIterator left = last-1;
   for(; left != first &&
         std::find(chars.begin(), chars.end(), *left) != chars.end(); --left);
-  return std::string(first, left+1);
+  return std::make_pair(first, left+1);
+}
+
+template<typename InputIterator>
+InputIterator lstripIter
+(InputIterator first, InputIterator last, char ch)
+{
+  for(; first != last && *first == ch; ++first);
+  return first;
+}
+
+template<typename InputIterator, typename InputIterator2>
+InputIterator lstripIter
+(InputIterator first, InputIterator last,
+ InputIterator2 cfirst, InputIterator2 clast)
+{
+  for(; first != last && std::find(cfirst, clast, *first) != clast; ++first);
+  return first;
+}
+
+template<typename InputIterator>
+InputIterator lstripIter
+(InputIterator first, InputIterator last)
+{
+  return lstripIter(first, last,
+                    DEFAULT_STRIP_CHARSET.begin(), DEFAULT_STRIP_CHARSET.end());
 }
 
 std::string strip
 (const std::string& str, const std::string& chars = DEFAULT_STRIP_CHARSET);
 
-bool startsWith(const std::string& target, const std::string& part);
-
-bool endsWith(const std::string& target, const std::string& part);
-
 std::string replace(const std::string& target, const std::string& oldstr, const std::string& newstr);
 
 std::string percentEncode(const unsigned char* target, size_t len);
 
 std::string percentEncode(const std::string& target);
 
+std::string percentEncodeMini(const std::string& target);
+
 bool inRFC3986ReservedChars(const char c);
 
 bool inRFC3986UnreservedChars(const char c);
 
 bool isUtf8(const std::string& str);
 
-std::string percentDecode(const std::string& target);
+std::string percentDecode
+(std::string::const_iterator first, std::string::const_iterator last);
 
 std::string torrentPercentEncode(const unsigned char* target, size_t len);
 
@@ -175,10 +234,30 @@ std::string toHex(const char* src, size_t len);
 
 std::string toHex(const std::string& src);
 
-// Converts hexadecimal ascii string 'src' into packed binary form and
-// return the result. If src is not well formed, then empty string is
-// returned.
-std::string fromHex(const std::string& src);
+unsigned int hexCharToUInt(unsigned char ch);
+
+// Converts hexadecimal ascii characters in [first, last) into packed
+// binary form and return the result. If characters in given range is
+// not well formed, then empty string is returned.
+template<typename InputIterator>
+std::string fromHex(InputIterator first, InputIterator last)
+{
+  std::string dest;
+  size_t len = last-first;
+  if(len%2) {
+    return dest;
+  }
+  for(; first != last; first += 2) {
+    unsigned char high = hexCharToUInt(*first);
+    unsigned char low = hexCharToUInt(*(first+1));
+    if(high == 255 || low == 255) {
+      dest.clear();
+      return dest;
+    }
+    dest += (high*16+low);
+  }
+  return dest;
+}
 
 FILE* openFile(const std::string& filename, const std::string& mode);
 
@@ -186,21 +265,17 @@ bool isPowerOf(int num, int base);
 
 std::string secfmt(time_t sec);
 
-int32_t parseInt(const std::string& s, int32_t base = 10);
-
-bool parseIntNoThrow(int32_t& result, const std::string& s, int base = 10);
+bool parseIntNoThrow(int32_t& res, const std::string& s, int base = 10);
+int32_t parseInt(const std::string& s, int base = 10);
 
+// Valid range: [0, INT32_MAX]
+bool parseUIntNoThrow(uint32_t& res, const std::string& s, int base = 10);
 uint32_t parseUInt(const std::string& s, int base = 10);
-  
-bool parseUIntNoThrow(uint32_t& result, const std::string& s, int base = 10);
 
-int64_t parseLLInt(const std::string& s, int32_t base = 10);
+bool parseLLIntNoThrow(int64_t& res, const std::string& s, int base = 10);
+int64_t parseLLInt(const std::string& s, int base = 10);
 
-bool parseLLIntNoThrow(int64_t& result, const std::string& s, int base = 10);
-
-uint64_t parseULLInt(const std::string& s, int base = 10);
-
-IntSequence parseIntRange(const std::string& src);
+void parseIntSegments(SegList<int>& sgl, const std::string& src);
 
 // Parses string which specifies the range of piece index for higher
 // priority and appends those indexes into result.  The input string
@@ -218,7 +293,7 @@ void parsePrioritizePieceRange
 (std::vector<size_t>& result, const std::string& src,
  const std::vector<SharedHandle<FileEntry> >& fileEntries,
  size_t pieceLength,
- uint64_t defaultSize = 1048576 /* 1MiB */);
+ off_t defaultSize = 1048576 /* 1MiB */);
 
 // Converts ISO/IEC 8859-1 string src to utf-8.
 std::string iso8859ToUtf8(const std::string& src);
@@ -246,37 +321,78 @@ int64_t getRealSize(const std::string& sizeWithUnit);
 
 std::string abbrevSize(int64_t size);
 
-template<typename InputIterator>
+template<typename InputIterator, typename Output>
 void toStream
-(InputIterator first, InputIterator last, std::ostream& os)
+(InputIterator first, InputIterator last, Output& os)
 {
-  os << _("Files:") << "\n";
-  os << "idx|path/length" << "\n";
-  os << "===+===========================================================================" << "\n";
+  os.printf("%s\n"
+            "idx|path/length\n"
+            "===+===========================================================================\n", _("Files:"));
   int32_t count = 1;
   for(; first != last; ++first, ++count) {
-    os << std::setw(3) << count << "|" << (*first)->getPath() << "\n";
-    os << "   |" << util::abbrevSize((*first)->getLength()) << "B ("
-       << util::uitos((*first)->getLength(), true) << ")\n";
-    os << "---+---------------------------------------------------------------------------" << "\n";
+    os.printf("%3d|%s\n"
+              "   |%sB (%s)\n"
+              "---+---------------------------------------------------------------------------\n",
+              count,
+              (*first)->getPath().c_str(),
+              util::abbrevSize((*first)->getLength()).c_str(),
+              util::uitos((*first)->getLength(), true).c_str());
   }
 }
 
 void sleep(long seconds);
 
 void usleep(long microseconds);
-  
-bool isNumber(const std::string& what);
+
+template<typename InputIterator>
+bool isNumber(InputIterator first, InputIterator last)
+{
+  if(first == last) {
+    return false;
+  }
+  for(; first != last; ++first) {
+    if('0' > *first || *first > '9') {
+      return false;
+    }
+  }
+  return true;
+}
+
+bool isAlpha(const char c);
 
 bool isDigit(const char c);
 
 bool isHexDigit(const char c);
 
 bool isHexDigit(const std::string& s);
-  
-bool isLowercase(const std::string& what);
-  
-bool isUppercase(const std::string& what);
+
+template<typename InputIterator>
+bool isLowercase(InputIterator first, InputIterator last)
+{
+  if(first == last) {
+    return false;
+  }
+  for(; first != last; ++first) {
+    if('a' > *first || *first > 'z') {
+      return false;
+    }
+  }
+  return true;
+}
+
+template<typename InputIterator>
+bool isUppercase(InputIterator first, InputIterator last)
+{
+  if(first == last) {
+    return false;
+  }
+  for(; first != last; ++first) {
+    if('A' > *first || *first > 'Z') {
+      return false;
+    }
+  }
+  return true;
+}
 
 /**
  * Converts alphabets to unsigned int, assuming alphabets as a base 26
@@ -326,29 +442,32 @@ std::string joinPath(InputIterator first, InputIterator last)
 
 // Parses INDEX=PATH format string. INDEX must be an unsigned
 // integer.
-std::map<size_t, std::string>::value_type
+std::pair<size_t, std::string>
 parseIndexPath(const std::string& line);
 
-std::map<size_t, std::string> createIndexPathMap(std::istream& i);
+std::vector<std::pair<size_t, std::string> > createIndexPaths(std::istream& i);
 
 /**
- * Take a string src which is a delimited list and add its elements
- * into result. result is stored in out.
+ * Take a string [first, last) which is a delimited list and add its
+ * elements into result as iterator pair. result is stored in out.
  */
-template<typename OutputIterator>
-OutputIterator split(const std::string& src, OutputIterator out,
-                     const std::string& delims, bool doStrip = false,
-                     bool allowEmpty = false)
+template<typename InputIterator, typename OutputIterator>
+OutputIterator splitIter
+(InputIterator first,
+ InputIterator last,
+ OutputIterator out,
+ char delim,
+ bool doStrip = false,
+ bool allowEmpty = false)
 {
-  std::string::const_iterator first = src.begin();
-  std::string::const_iterator last = src.end();
-  for(std::string::const_iterator i = first; i != last;) {
-    std::string::const_iterator j = i;
-    for(; j != last &&
-          std::find(delims.begin(), delims.end(), *j) == delims.end(); ++j);
-    std::string t = doStrip?util::stripIter(i, j):std::string(i, j);
-    if(allowEmpty || !t.empty()) {
-      *out++ = t;
+  for(InputIterator i = first; i != last;) {
+    InputIterator j = std::find(i, last, delim);
+    std::pair<InputIterator, InputIterator> p(i, j);
+    if(doStrip) {
+      p = stripIter(i, j);
+    }
+    if(allowEmpty || p.first != p.second) {
+      *out++ = p;
     }
     i = j;
     if(j != last) {
@@ -356,14 +475,234 @@ OutputIterator split(const std::string& src, OutputIterator out,
     }
   }
   if(allowEmpty &&
-     (src.empty() ||
-      std::find(delims.begin(), delims.end(),
-                src[src.size()-1]) != delims.end())) {
-    *out++ = A2STR::NIL;
+     (first == last || *(last-1) == delim)) {
+    *out++ = std::make_pair(last, last);
   }
   return out;
 }
 
+template<typename InputIterator, typename OutputIterator>
+OutputIterator splitIterM
+(InputIterator first,
+ InputIterator last,
+ OutputIterator out,
+ const char* delims,
+ bool doStrip = false,
+ bool allowEmpty = false)
+{
+  size_t numDelims = strlen(delims);
+  const char* dlast = delims+numDelims;
+  for(InputIterator i = first; i != last;) {
+    InputIterator j = i;
+    for(; j != last && std::find(delims, dlast, *j) == dlast; ++j);
+    std::pair<InputIterator, InputIterator> p(i, j);
+    if(doStrip) {
+      p = stripIter(i, j);
+    }
+    if(allowEmpty || p.first != p.second) {
+      *out++ = p;
+    }
+    i = j;
+    if(j != last) {
+      ++i;
+    }
+  }
+  if(allowEmpty &&
+     (first == last ||
+      std::find(delims, dlast, *(last-1)) != dlast)) {
+    *out++ = std::make_pair(last, last);
+  }
+  return out;
+}
+
+template<typename InputIterator, typename OutputIterator>
+OutputIterator split
+(InputIterator first,
+ InputIterator last,
+ OutputIterator out,
+ char delim,
+ bool doStrip = false,
+ bool allowEmpty = false)
+{
+  for(InputIterator i = first; i != last;) {
+    InputIterator j = std::find(i, last, delim);
+    std::pair<InputIterator, InputIterator> p(i, j);
+    if(doStrip) {
+      p = stripIter(i, j);
+    }
+    if(allowEmpty || p.first != p.second) {
+      *out++ = std::string(p.first, p.second);
+    }
+    i = j;
+    if(j != last) {
+      ++i;
+    }
+  }
+  if(allowEmpty &&
+     (first == last || *(last-1) == delim)) {
+    *out++ = std::string(last, last);
+  }
+  return out;
+}
+
+template<typename InputIterator1, typename InputIterator2>
+bool streq
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 != last2-first2) {
+    return false;
+  }
+  return std::equal(first1, last1, first2);
+}
+
+template<typename InputIterator>
+bool streq(InputIterator first, InputIterator last, const char* b)
+{
+  for(; first != last && *b != '\0'; ++first, ++b) {
+    if(*first != *b) {
+      return false;
+    }
+  }
+  return first == last && *b == '\0';
+}
+
+struct CaseCmp {
+  bool operator()(char lhs, char rhs) const
+  {
+    if('A' <= lhs && lhs <= 'Z') {
+      lhs += 'a'-'A';
+    }
+    if('A' <= rhs && rhs <= 'Z') {
+      rhs += 'a'-'A';
+    }
+    return lhs == rhs;
+  }
+};
+
+template<typename InputIterator1, typename InputIterator2>
+InputIterator1 strifind
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  return std::search(first1, last1, first2, last2, CaseCmp());
+}
+
+template<typename InputIterator1, typename InputIterator2>
+bool strieq
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 != last2-first2) {
+    return false;
+  }
+  return std::equal(first1, last1, first2, CaseCmp());
+}
+
+template<typename InputIterator>
+bool strieq(InputIterator first, InputIterator last, const char* b)
+{
+  CaseCmp cmp;
+  for(; first != last && *b != '\0'; ++first, ++b) {
+    if(!cmp(*first, *b)) {
+      return false;
+    }
+  }
+  return first == last && *b == '\0';
+}
+
+template<typename InputIterator1, typename InputIterator2>
+bool startsWith
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 < last2-first2) {
+    return false;
+  }
+  return std::equal(first2, last2, first1);
+}
+
+template<typename InputIterator>
+bool startsWith(InputIterator first, InputIterator last, const char* b)
+{
+  for(; first != last && *b != '\0'; ++first, ++b) {
+    if(*first != *b) {
+      return false;
+    }
+  }
+  return *b == '\0';
+}
+
+bool startsWith(const std::string& a, const char* b);
+bool startsWith(const std::string& a, const std::string& b);
+
+template<typename InputIterator1, typename InputIterator2>
+bool istartsWith
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 < last2-first2) {
+    return false;
+  }
+  return std::equal(first2, last2, first1, CaseCmp());
+}
+
+template<typename InputIterator>
+bool istartsWith(InputIterator first, InputIterator last, const char* b)
+{
+  CaseCmp cmp;
+  for(; first != last && *b != '\0'; ++first, ++b) {
+    if(!cmp(*first, *b)) {
+      return false;
+    }
+  }
+  return *b == '\0';
+}
+
+bool istartsWith(const std::string& a, const char* b);
+
+template<typename InputIterator1, typename InputIterator2>
+bool endsWith
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 < last2-first2) {
+    return false;
+  }
+  return std::equal(first2, last2, last1-(last2-first2));
+}
+
+bool endsWith(const std::string& a, const char* b);
+bool endsWith(const std::string& a, const std::string& b);
+
+template<typename InputIterator1, typename InputIterator2>
+bool iendsWith
+(InputIterator1 first1,
+ InputIterator1 last1,
+ InputIterator2 first2,
+ InputIterator2 last2)
+{
+  if(last1-first1 < last2-first2) {
+    return false;
+  }
+  return std::equal(first2, last2, last1-(last2-first2), CaseCmp());
+}
+
+bool iendsWith(const std::string& a, const char* b);
+bool iendsWith(const std::string& a, const std::string& b);
+
 void generateRandomData(unsigned char* data, size_t length);
 
 // Saves data to file whose name is filename. If overwrite is true,
@@ -407,12 +746,13 @@ bool detectDirTraversal(const std::string& s);
 // '_': '"', '*', ':', '<', '>', '?', '\', '|'.
 std::string escapePath(const std::string& s);
 
-// Stores network address of numeric IPv4 address ip using CIDR bits
-// into in.  On success, returns true. Otherwise returns false.
-bool getCidrPrefix(struct in_addr& in, const std::string& ip, int bits);
-
-// Returns true if ip1 and ip2 are in the same CIDR block.
-bool inSameCidrBlock(const std::string& ip1, const std::string& ip2, int bits);
+// Returns true if ip1 and ip2 are in the same CIDR block.  ip1 and
+// ip2 must be numeric IPv4 or IPv6 address. If either of them or both
+// of them is not valid numeric address, then returns false. bits is
+// prefix bits. If bits is out of range, then bits is set to the
+// length of binary representation of the address*8.
+bool inSameCidrBlock
+(const std::string& ip1, const std::string& ip2, size_t bits);
 
 void removeMetalinkContentTypes(const SharedHandle<RequestGroup>& group);
 void removeMetalinkContentTypes(RequestGroup* group);
@@ -420,11 +760,11 @@ void removeMetalinkContentTypes(RequestGroup* group);
 // No throw
 void executeHookByOptName
 (const SharedHandle<RequestGroup>& group, const Option* option,
- const std::string& opt);
+ const Pref* pref);
 
 // No throw
 void executeHookByOptName
-(const RequestGroup* group, const Option* option, const std::string& opt);
+(const RequestGroup* group, const Option* option, const Pref* pref);
 
 std::string createSafePath(const std::string& dir, const std::string& filename);
 
@@ -467,24 +807,27 @@ nextParam
         parmnameLast = last;
       }
     }
-    std::string tname, tvalue;
+    std::pair<std::string::const_iterator,
+              std::string::const_iterator> namep;
+    std::pair<std::string::const_iterator,
+              std::string::const_iterator> valuep;
     if(parmnameFirst == parmnameLast) {
       if(!eqFound) {
         parmnameFirst = first;
         parmnameLast = last;
-        tname = util::stripIter(parmnameFirst, parmnameLast);
+        namep = stripIter(parmnameFirst, parmnameLast);
       }
     } else {
       first = parmnameLast+1;
-      tname = util::stripIter(parmnameFirst, parmnameLast);
-      tvalue = util::stripIter(first, last);
+      namep = stripIter(parmnameFirst, parmnameLast);
+      valuep = stripIter(first, last);
     }
     if(last != end) {
       ++last;
     }
-    if(!tname.empty()) {
-      name.swap(tname);
-      value.swap(tvalue);
+    if(namep.first != namep.second) {
+      name.assign(namep.first, namep.second);
+      value.assign(valuep.first, valuep.second);
       return std::make_pair(last, true);
     }
     first = last;
@@ -492,6 +835,23 @@ nextParam
   return std::make_pair(end, false);
 }
 
+template<typename T>
+SharedHandle<T> copy(const SharedHandle<T>& a)
+{
+  return SharedHandle<T>(new T(*a.get()));
+}
+
+// This is a bit different from cookie_helper::domainMatch().  If
+// hostname is numeric host, then returns true if domain == hostname.
+// That is if domain starts with ".", then returns true if domain is a
+// suffix of hostname.  If domain does not start with ".", then
+// returns true if domain == hostname.  Otherwise returns true.
+// For example,
+//
+// * noProxyDomainMatch("aria2.sf.net", ".sf.net") returns true.
+// * noProxyDomainMatch("sf.net", ".sf.net") returns false.
+bool noProxyDomainMatch(const std::string& hostname, const std::string& domain);
+
 } // namespace util
 
 } // namespace aria2
