@@ -35,13 +35,13 @@ void AuthConfigFactoryTest::testCreateAuthConfig_http()
   req->setUri("http://localhost/download/aria2-1.0.0.tar.bz2");
 
   Option option;
-  option.put(PREF_NO_NETRC, V_FALSE);
-  option.put(PREF_HTTP_AUTH_CHALLENGE, V_TRUE);
+  option.put(PREF_NO_NETRC, A2_V_FALSE);
+  option.put(PREF_HTTP_AUTH_CHALLENGE, A2_V_TRUE);
 
   AuthConfigFactory factory;
 
   // without auth info
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
   // with Netrc
   SharedHandle<Netrc> netrc(new Netrc());
@@ -55,9 +55,9 @@ void AuthConfigFactoryTest::testCreateAuthConfig_http()
   factory.setNetrc(netrc);
 
   // not activated
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
-  CPPUNIT_ASSERT(factory.activateBasicCred("localhost", "/", &option));
+  CPPUNIT_ASSERT(factory.activateBasicCred("localhost", 80, "/", &option));
 
   CPPUNIT_ASSERT_EQUAL(std::string("localhostuser:localhostpass"),
                        factory.createAuthConfig(req, &option)->getAuthText());
@@ -65,17 +65,17 @@ void AuthConfigFactoryTest::testCreateAuthConfig_http()
   // See default token in netrc is ignored.
   req->setUri("http://mirror/");
 
-  CPPUNIT_ASSERT(!factory.activateBasicCred("mirror", "/", &option));
+  CPPUNIT_ASSERT(!factory.activateBasicCred("mirror", 80, "/", &option));
 
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
   // with Netrc + user defined
   option.put(PREF_HTTP_USER, "userDefinedUser");
   option.put(PREF_HTTP_PASSWD, "userDefinedPassword");
 
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
-  CPPUNIT_ASSERT(factory.activateBasicCred("mirror", "/", &option));
+  CPPUNIT_ASSERT(factory.activateBasicCred("mirror", 80, "/", &option));
 
   CPPUNIT_ASSERT_EQUAL(std::string("userDefinedUser:userDefinedPassword"),
                        factory.createAuthConfig(req, &option)->getAuthText());
@@ -92,12 +92,12 @@ void AuthConfigFactoryTest::testCreateAuthConfig_httpNoChallenge()
   req->setUri("http://localhost/download/aria2-1.0.0.tar.bz2");
 
   Option option;
-  option.put(PREF_NO_NETRC, V_FALSE);
+  option.put(PREF_NO_NETRC, A2_V_FALSE);
 
   AuthConfigFactory factory;
 
   // without auth info
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
   // with Netrc
   SharedHandle<Netrc> netrc(new Netrc());
@@ -117,7 +117,7 @@ void AuthConfigFactoryTest::testCreateAuthConfig_httpNoChallenge()
   // See default token in netrc is ignored.
   req->setUri("http://mirror/");
 
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
   // with Netrc + user defined
   option.put(PREF_HTTP_USER, "userDefinedUser");
@@ -138,7 +138,7 @@ void AuthConfigFactoryTest::testCreateAuthConfig_ftp()
   req->setUri("ftp://localhost/download/aria2-1.0.0.tar.bz2");
 
   Option option;
-  option.put(PREF_NO_NETRC, V_FALSE);
+  option.put(PREF_NO_NETRC, A2_V_FALSE);
 
   AuthConfigFactory factory;
 
@@ -155,12 +155,12 @@ void AuthConfigFactoryTest::testCreateAuthConfig_ftp()
                        factory.createAuthConfig(req, &option)->getAuthText());
 
   // disable Netrc
-  option.put(PREF_NO_NETRC, V_TRUE);
+  option.put(PREF_NO_NETRC, A2_V_TRUE);
   CPPUNIT_ASSERT_EQUAL(std::string("anonymous:ARIA2USER@"),
                        factory.createAuthConfig(req, &option)->getAuthText());
 
   // with Netrc + user defined
-  option.put(PREF_NO_NETRC, V_FALSE);
+  option.put(PREF_NO_NETRC, A2_V_FALSE);
   option.put(PREF_FTP_USER, "userDefinedUser");
   option.put(PREF_FTP_PASSWD, "userDefinedPassword");
   CPPUNIT_ASSERT_EQUAL(std::string("userDefinedUser:userDefinedPassword"),
@@ -198,26 +198,31 @@ void AuthConfigFactoryTest::testCreateAuthConfig_ftp()
 void AuthConfigFactoryTest::testUpdateBasicCred()
 {
   Option option;
-  option.put(PREF_NO_NETRC, V_FALSE);
-  option.put(PREF_HTTP_AUTH_CHALLENGE, V_TRUE);
+  option.put(PREF_NO_NETRC, A2_V_FALSE);
+  option.put(PREF_HTTP_AUTH_CHALLENGE, A2_V_TRUE);
 
   AuthConfigFactory factory;
 
   factory.updateBasicCred
-    (AuthConfigFactory::BasicCred("myname", "mypass", "localhost", "/", true));
+    (AuthConfigFactory::BasicCred("myname", "mypass", "localhost", 80, "/", true));
   factory.updateBasicCred
-    (AuthConfigFactory::BasicCred("price","j38jdc","localhost","/download", true));
+    (AuthConfigFactory::BasicCred("price", "j38jdc", "localhost", 80, "/download", true));
   factory.updateBasicCred
-    (AuthConfigFactory::BasicCred("alice","ium8","localhost","/documents", true));
+    (AuthConfigFactory::BasicCred("soap", "planB", "localhost", 80, "/download/beta", true));
   factory.updateBasicCred
-    (AuthConfigFactory::BasicCred("jack", "jackx","mirror", "/doc", true));
+    (AuthConfigFactory::BasicCred("alice", "ium8", "localhost", 80, "/documents", true));
+  factory.updateBasicCred
+    (AuthConfigFactory::BasicCred("jack", "jackx", "mirror", 80, "/doc", true));
 
   SharedHandle<Request> req(new Request());
   req->setUri("http://localhost/download/v2.6/Changelog");
-  
   CPPUNIT_ASSERT_EQUAL(std::string("price:j38jdc"),
                        factory.createAuthConfig(req, &option)->getAuthText());
 
+  req->setUri("http://localhost/download/beta/v2.7/Changelog");
+  CPPUNIT_ASSERT_EQUAL(std::string("soap:planB"),
+                       factory.createAuthConfig(req, &option)->getAuthText());
+
   req->setUri("http://localhost/documents/reference.html");
   CPPUNIT_ASSERT_EQUAL(std::string("alice:ium8"),
                        factory.createAuthConfig(req, &option)->getAuthText());
@@ -230,11 +235,14 @@ void AuthConfigFactoryTest::testUpdateBasicCred()
   CPPUNIT_ASSERT_EQUAL(std::string("myname:mypass"),
                        factory.createAuthConfig(req, &option)->getAuthText());
 
+  req->setUri("http://localhost:8080/doc/readme.txt");
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
+
   req->setUri("http://local/");
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 
   req->setUri("http://mirror/");
-  CPPUNIT_ASSERT(factory.createAuthConfig(req, &option).isNull());
+  CPPUNIT_ASSERT(!factory.createAuthConfig(req, &option));
 }
 
 } // namespace aria2
