@@ -1,5 +1,5 @@
-#ifndef _D_MOCK_DHT_MESSAGE_CALLBACK_H_
-#define _D_MOCK_DHT_MESSAGE_CALLBACK_H_
+#ifndef D_MOCK_DHT_MESSAGE_CALLBACK_H
+#define D_MOCK_DHT_MESSAGE_CALLBACK_H
 
 #include "DHTMessageCallback.h"
 
@@ -24,4 +24,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_MOCK_DHT_MESSAGE_CALLBACK_H_
+#endif // D_MOCK_DHT_MESSAGE_CALLBACK_H
